package Dancer2::Plugin::Auth::Extensible::Provider::Example;

=head1 NAME 

Dancer2::Plugin::Auth::Extensible::Example

=head1 DESCRIPTION

See L<Dancer2::Plugin::Auth::Extensible::Config> for an example provider.
See also L<Dancer2::Plugin::Auth::Extensible::Role::Provider>.

=cut

our $VERSION = '0.706';

1;
